<?php

declare(strict_types=1);

namespace Drupal\component_library\Entity;

use Drupal\Core\Cache\Cache;
use Drupal\Core\Config\Entity\ConfigEntityBase;
use Drupal\Core\Entity\EntityStorageInterface;

/**
 * Defines the component_library_asset entity type.
 *
 * @ConfigEntityType(
 *   id = "component_library_asset",
 *   label = @Translation("Component Library Asset"),
 *   label_collection = @Translation("Component Library Assets"),
 *   label_singular = @Translation("Component Library Asset"),
 *   label_plural = @Translation("Component Library Assets"),
 *   label_count = @PluralTranslation(
 *     singular = "@count Component Library Asset",
 *     plural = "@count Component Library Assets",
 *   ),
 *   handlers = {
 *     "list_builder" =
 *   "Drupal\component_library\ComponentLibraryAssetListBuilder",
 *     "form" = {
 *       "add" = "Drupal\component_library\Form\ComponentLibraryAssetForm",
 *       "edit" = "Drupal\component_library\Form\ComponentLibraryAssetForm",
 *       "delete" = "Drupal\Core\Entity\EntityDeleteForm"
 *     }
 *   },
 *   config_prefix = "component_library_asset",
 *   admin_permission = "administer component library assets",
 *   links = {
 *     "collection" = "/admin/structure/component-library/assets",
 *     "add-form" = "/admin/structure/component-library/assets/add",
 *     "edit-form" = "/admin/structure/component-library/assets/{component_library_asset}",
 *     "delete-form" = "/admin/structure/component-library/assets/{component_library_asset}/delete"
 *   },
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "label",
 *     "uuid" = "uuid"
 *   },
 *   config_export = {
 *     "id",
 *     "label",
 *     "theme",
 *     "css",
 *     "js",
 *     "paths",
 *   }
 * )
 */
final class ComponentLibraryAsset extends ConfigEntityBase {

  /**
   * The component_library_asset ID.
   */
  protected string $id;

  /**
   * The component_library_asset label.
   */
  protected string $label;

  /**
   * The theme the component_library_asset entity is associated with.
   */
  protected string $theme;

  /**
   * The css code of the library.
   */
  protected array $css = [];

  /**
   * The javascript code of the library.
   */
  protected array $js = [];

  /**
   * The library files that were generated.
   */
  protected array $paths = [];

  /**
   * {@inheritdoc}
   */
  public static function postDelete(EntityStorageInterface $storage, array $entities): void {
    /** @var \Drupal\component_library\Entity\ComponentLibraryAsset $entity */
    foreach ($entities as $entity) {
      self::clearFiles($entity);
    }
  }

  /**
   * Clears existing library files and ensures working if aggregation is active.
   *
   * @param \Drupal\component_library\Entity\ComponentLibraryAsset $asset
   *   The component_library_asset to clean up files for.
   */
  public static function clearFiles(ComponentLibraryAsset $asset): void {
    Cache::invalidateTags(['library_info']);
    $file_system = \Drupal::service('file_system');
    foreach (['js', 'css'] as $asset_type) {
      $aggregation_enabled = \Drupal::config('system.performance')->get($asset_type . '.preprocess');
      if ($aggregation_enabled) {
        $file_system->deleteRecursive('public://' . $asset_type . '/');
      }
    }
    $path = 'public://component_library_assets/' . $asset->get('id');
    $file_system->deleteRecursive($path);
  }

}
