<?php

declare(strict_types=1);

namespace Drupal\component_library\Form;

use Drupal\component_library\Entity\ComponentLibraryAsset;
use Drupal\Core\Asset\LibraryDiscoveryInterface;
use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Extension\Extension;
use Drupal\Core\Extension\ThemeHandlerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * The configuration form for component_library_asset entities.
 */
final class ComponentLibraryAssetForm extends EntityForm {

  /**
   * The theme handler.
   */
  protected ThemeHandlerInterface $themeHandler;

  /**
   * The Drupal file system.
   */
  protected FileSystemInterface $fileSystem;

  /**
   * The library discovery.
   */
  protected LibraryDiscoveryInterface $libraryDiscovery;

  /**
   * Constructs a new ComponentLibraryAssetForm.
   *
   * @param \Drupal\Core\Extension\ThemeHandlerInterface $theme_handler
   *   The theme handler.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The Drupal file system.
   * @param \Drupal\Core\Asset\LibraryDiscoveryInterface $library_discovery
   *   The library discovery.
   */
  public function __construct(ThemeHandlerInterface $theme_handler, FileSystemInterface $file_system, LibraryDiscoveryInterface $library_discovery) {
    $this->themeHandler = $theme_handler;
    $this->fileSystem = $file_system;
    $this->libraryDiscovery = $library_discovery;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new self(
      $container->get('theme_handler'),
      $container->get('file_system'),
      $container->get('library.discovery')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state): array {
    $form = parent::form($form, $form_state);

    $form['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#maxlength' => 255,
      '#default_value' => $this->entity->label(),
      '#description' => $this->t('Label for the component_library_asset.'),
      '#required' => TRUE,
    ];

    $form['id'] = [
      '#type' => 'machine_name',
      '#default_value' => $this->entity->id(),
      '#machine_name' => [
        'exists' => '\Drupal\component_library\Entity\ComponentLibraryAsset::load',
      ],
      '#disabled' => !$this->entity->isNew(),
    ];

    $themes = $this->themeHandler->listInfo();
    $themes = \array_filter($themes, fn (Extension $theme) => $this->themeHandler->hasUi($theme->getName()));
    $themes = \array_combine(\array_keys($themes), \array_keys($themes));
    $form['theme'] = [
      '#type' => 'select',
      '#title' => $this->t('Theme'),
      '#empty_option' => $this->t('- Select -'),
      '#default_value' => $this->entity->get('theme') ?? $form_state->getValue('theme') ?? '',
      '#options' => $themes,
      '#required' => TRUE,
    ];

    if (!$form_state->get('library_files')) {
      $form_state->set('library_files', [
        'css' => $this->getFilesDefaultValue('css', $form_state),
        'js' => $this->getFilesDefaultValue('js', $form_state),
      ]);
    }
    foreach (\array_keys($form_state->get('library_files')) as $asset_type) {
      $default_files = $form_state->get('library_files')[$asset_type];
      $this->addMultiValueField($form, $asset_type, $default_files);
    }

    $form['#attached']['library'][] = 'codemirror_editor/editor';
    return $form;
  }

  /**
   * AJAX callback to add/remove a multivalue element on the CSS fieldset.
   */
  public function ajaxCssFiles(array $form, FormStateInterface $form_state): array {
    return $form['css'];
  }

  /**
   * AJAX callback to add/remove a multivalue element on the JS fieldset.
   */
  public function ajaxJsFiles(array $form, FormStateInterface $form_state): array {
    return $form['js'];
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    $library_files = $form_state->get('library_files');
    $trigger = $form_state->getTriggeringElement();
    if (isset($trigger['#name']) && \strpos($trigger['#name'], 'add_button') !== FALSE) {
      $asset_type = \substr($trigger['#name'], \strlen('add_button_'));
      $library_files[$asset_type][] = ['name' => '', 'code' => ''];
      $form_state->set('files_altered', [$asset_type => TRUE]);
    }
    if (isset($trigger['#parents'][1]) && isset($trigger['#name']) && \strpos($trigger['#name'], 'remove_button') !== FALSE) {
      $asset_type = \substr($trigger['#name'], \strlen('remove_button_' . $trigger['#parents'][1]) + 1);
      \array_splice($library_files[$asset_type], $trigger['#parents'][1], 1);
      $form_state->set('files_altered', [$asset_type => TRUE]);
    }
    $form_state->set('library_files', $library_files);
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    // Re-create the files on disk and make sure they are read when aggregation
    // is enabled.
    ComponentLibraryAsset::clearFiles($this->entity);
    $this->generateLibraryFiles($form_state);
    $result = parent::save($form, $form_state);

    $message_args = ['%label' => $this->entity->label()];
    $message = $result == \SAVED_NEW
      ? $this->t('Created new component_library_asset %label.', $message_args)
      : $this->t('Updated component_library_asset %label.', $message_args);
    $this->messenger()->addStatus($message);
    $form_state->setRedirectUrl($this->entity->toUrl('collection'));
    return $result;
  }

  /**
   * {@inheritdoc}
   */
  protected function copyFormValuesToEntity(EntityInterface $entity, array $form, FormStateInterface $form_state): void {
    foreach (\array_keys($form_state->get('library_files')) as $asset_type) {
      $value = \array_filter($form_state->getValue($asset_type), static fn ($element) => !($element instanceof TranslatableMarkup) && !empty($element['name']) && !empty($element['code']));
      $form_state->setValue($asset_type, $value);
    }
    parent::copyFormValuesToEntity($entity, $form, $form_state);
  }

  /**
   * Determines default value for the given multivalue field.
   *
   * @param string $asset_type
   *   The file type/field to get the default value for.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state to pull values from if file values were already changed.
   *
   * @return array
   *   Array of names and codes for the given file type/field.
   */
  private function getFilesDefaultValue($asset_type, FormStateInterface $form_state): array {
    return empty($form_state->get('files_altered')[$asset_type])
      ? $this->getEntity()->get($asset_type)
      : $form_state->getValue($asset_type);
  }

  /**
   * Adds multivalue field consisting of name and code subfields to given form.
   *
   * @param array $form
   *   The form to add the fields to.
   * @param string $asset_type
   *   The asset type of the field, either css or js.
   * @param array $default_files
   *   The default values for the fields.
   */
  private function addMultiValueField(array &$form, $asset_type, array $default_files): void {
    $ajax_callback = 'ajax' . \ucfirst($asset_type) . 'Files';
    $form[$asset_type] = [
      '#type' => 'details',
      '#open' => TRUE,
      '#tree' => TRUE,
      '#title' => $this->t('@asset_type Files', ['@asset_type' => \strtoupper($asset_type)]),
      '#prefix' => '<div id="' . $asset_type . '-container">',
      '#suffix' => '</div>',
      'add_button_' . $asset_type => [
        '#type' => 'submit',
        '#value' => $this->t('Add file'),
        '#name' => 'add_button_' . $asset_type,
        '#weight' => 999,
        '#limit_validation_errors' => TRUE,
        '#executes_submit_callback' => FALSE,
        '#ajax' => [
          'callback' => [$this, $ajax_callback],
          'wrapper' => $asset_type . '-container',
          'effect' => 'fade',
        ],
      ],
    ];

    foreach ($default_files as $delta => $values) {
      $form[$asset_type][$delta] = [
        '#type' => 'container',
      ];

      $form[$asset_type][$delta]['name'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Library Name'),
        '#maxlength' => 255,
        '#default_value' => $values['name'],
        '#description' => $this->t('Name for the library asset file.'),
      ];

      $form[$asset_type][$delta]['code'] = [
        '#type' => 'codemirror',
        '#codemirror' => [
          'autoCloseTags' => TRUE,
          'buttons' => [
            'undo',
            'redo',
            'enlarge',
            'shrink',
          ],
          'foldGutter' => TRUE,
          'lineNumbers' => TRUE,
          'lineWrapping' => TRUE,
          'styleActiveLine' => TRUE,
          'toolbar' => TRUE,
          'lineSeparator' => "\n",
          'mode' => $asset_type == 'js' ? 'javascript' : $asset_type,
        ],
        '#title' => $this->t('Code'),
        '#default_value' => $values['code'],
      ];

      $form[$asset_type][$delta]['remove_button'] = [
        '#type' => 'submit',
        '#value' => $this->t('Remove file'),
        '#name' => 'remove_button_' . $delta . '_' . $asset_type,
        '#limit_validation_errors' => TRUE,
        '#executes_submit_callback' => FALSE,
        '#ajax' => [
          'callback' => [$this, $ajax_callback],
          'wrapper' => $asset_type . '-container',
          'effect' => 'fade',
        ],
      ];
    }
  }

  /**
   * Writes the code saved in the css and js fields into the public files dir.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state after submission of the form.
   */
  private function generateLibraryFiles(FormStateInterface $form_state): void {
    $paths = [];
    foreach (\array_keys($form_state->get('library_files')) as $asset_type) {
      $dir = 'public://component_library_assets/' . $form_state->getValue('id') . '/' . $asset_type;
      $this->fileSystem->prepareDirectory($dir, FileSystemInterface::CREATE_DIRECTORY | FileSystemInterface::MODIFY_PERMISSIONS);
      foreach ($form_state->getValue($asset_type) as $library_file) {
        $file_name = $library_file['name'];
        $file_path = "$dir/$file_name.$asset_type";
        $path = $this->fileSystem->saveData(
          $library_file['code'],
          $file_path,
          FileSystemInterface::EXISTS_REPLACE
        );
        $paths[] = [
          'type' => $asset_type,
          'path' => $path,
        ];
      }
    }
    $this->getEntity()->set('paths', $paths);
    $this->libraryDiscovery->clearCachedDefinitions();
  }

}
